@@ -284,7 +284,9 @@ class ContractKssView(KSSView):
                          'contract_viewlet_duration_no_integer',
                          'contract_viewlet_hours_no_integer',
                          'contract_viewlet_workdays_no_integer',
-                         'contract_viewlet_incorrect_workdays']:
+                         'contract_viewlet_incorrect_workdays',
+                         'contract_viewlet_spread_to_high',
+                         'contract_viewlet_more_than_24']:
             selector = core.getHtmlIdSelector(errorMsg)
             core.setAttribute(selector, 'class', 'dont-show')
 
@@ -328,6 +330,32 @@ class ContractKssView(KSSView):
             if workdays:
                 errors.append('contract_viewlet_workdays_no_integer')
 
+        # We check that number of hours specified is correct.
+        if 'contract_form_working_schedule' in form:
+            total = 0
+            if form['contract_form_working_schedule'] == 'manual':
+                rows = ['odd']
+            elif form['contract_form_working_schedule'] == 'manual_oddeven':
+                rows = ['odd', 'even']
+            else:
+                rows = []
+
+            for row in rows:
+                for day in range(0, 7):
+                    try:
+                        field = float(form['schedule_' + row + '_' + str(day)])
+                    except:
+                        field = 0
+                    if field in form:                           
+                        total += field
+                        if field > 24:
+                            errors.append('contract_viewlet_more_than_24')
+
+            if rows:
+                total = total / len(rows)
+                if total > int(number_hours):
+                    errors.append('contract_viewlet_spread_to_high')
+
         # We display the error messages.
         for errorMsg in errors:
             selector = core.getHtmlIdSelector(errorMsg)
@@ -378,6 +406,10 @@ class ContractKssView(KSSView):
         contract._renameAfterCreation()
         notify(ObjectInitializedEvent(contract))
 
+        # Adds informations for hours spread.
+        if 'contract_form_working_schedule' in form:
+            contract.hour_spread.update_from_form(form)
+
         # Shows status message
         if is_contract:
             message = translate(_(u'msg_contract_added',
@@ -391,3 +423,38 @@ class ContractKssView(KSSView):
      
 
         self.show_list(hide_status=False)
+
+    @kssaction
+    def change_schedule_table(self, value):
+        """ Show or hide the schedule table.
+        """
+
+        if value == 'auto':
+            css = 'dont-show'
+        elif value in ['manual', 'manual_oddeven']:
+            css = 'listing'
+        else:
+            return
+        
+        core = self.getCommandSet('core')
+        selector = core.getHtmlIdSelector('contract_viewlet_manual_schedule')
+        core.setAttribute(selector, 'class', css)
+
+        if not value == 'auto':
+            if value == 'manual':
+                css = 'dont-show'
+                first_row = _(u'label_normal', u'normal')
+            else:
+                css = ''
+                first_row = _(u'label_odd', u'odd')
+            
+            # We hide the second row which is useless.
+            selector = core.getHtmlIdSelector(
+                'contract_viewlet_manual_schedule_even')
+            core.setAttribute(selector, 'class', css)
+
+            selector = core.getHtmlIdSelector(
+                'contract_viewlet_manual_schedule_odd_first_row')
+            core.replaceInnerHTML(selector,
+                                  translate(first_row,
+                                            target_language=self.lang))
